#================================================
#
#    wallpapoz_system.py - Wallpapoz
#    Copyright (C) 2007 Vajrasky Akbar Kok <arjuna@vajrasky.net>
#
#================================================
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#================================================

## wallpapoz_system.py -- finds current desktop and changes wallpaper
# achieve goal by calling external program

import subprocess
import os
import sys
import string
import signal

class WallpapozSystem:

  # Define static variable to save the initial value of
  # workspaces
  static_finding_total_workspaces_called_p = False
  static_total_workspaces = 0

  def __init__(self):
    self.finding_screen_resolution()
    self.finding_total_workspaces()
    self.finding_desktop_environment()

    # Check if total_workspaces changed only here
    if WallpapozSystem.static_total_workspaces != self.total_workspaces:
      print ("daemon_wallpapoz: the number of total workspaces changed during initialization process.")
      print ("daemon_wallpapoz: respawning daemon_wallpapoz.")
      self.respawn_system()

  def exec_cmd_under_X(self, cmd):
    try:
      p = subprocess.Popen(
        cmd,
        stdout = subprocess.PIPE,
        stderr = subprocess.PIPE,
        shell = True
        )
    except OSError:
      print ("daemon_wallpapoz: fork failed for %s, exiting." %cmd)
      sys.exit(1)

    pstdout = p.stdout.read()
    pstderr = p.stderr.read()
    ret = p.wait()
    # FIXME
    pstdout_str = pstdout.decode('utf-8')

    kill_daemon = False

    if (ret != 0) and (ret & 0xFF):
        print ("daemon_wallpapoz: %s returned status %i." %(cmd, ret))
        print ("daemon_wallpapoz: error message: %s" %pstderr)
        kill_daemon = True

    if (cmd.find('xprop') >= 0) and (pstdout_str.find('no such atom') >= 0):
        kill_daemon = True
    if (cmd.find('xprop') >= 0) and (pstdout_str.find('not') >= 0):
        kill_daemon = True

    if kill_daemon:
        ## No X resource available, kill daemon_wallpapoz
        print ("daemon_wallpapoz: X resource seems no longer available.")
        print ("daemon_wallpapoz: killing daemon_wallpapoz.")
        sys.exit(1)

    #FIXME
    return pstdout_str

  def set_style(self, style):
    self.wallpaper_style = style

  ## class method to find monitor resolution
  def finding_screen_resolution(self):
    raw_resolution = self.exec_cmd_under_X('xwininfo -root')
    start_width = raw_resolution.find('Width')
    end_width = raw_resolution.find('\n',start_width)
    start_height = raw_resolution.find('Height')
    end_height = raw_resolution.find('\n', start_height)
    self.screen_width = int(raw_resolution[start_width+7:end_width])
    self.screen_height = int(raw_resolution[start_height+8:end_height])

  ## class method to find which desktop environment user uses
  def finding_desktop_environment(self):
    raw_window_id = self.exec_cmd_under_X('xprop -root _NET_SUPPORTING_WM_CHECK')
    window_id = raw_window_id[46:raw_window_id.find("\n")]
    raw_wm_name = self.exec_cmd_under_X('xprop -id ' + window_id + ' 8s _NET_WM_NAME')
    wm_name = raw_wm_name[29:raw_wm_name.rfind('"')]
    # default is Gnome3.
    self.window_manager = 'Gnome3'
    if wm_name == 'Openbox':
      try:
        env = os.environ['XDG_CURRENT_DESKTOP']
        if env == 'LXDE':
          self.window_manager = 'LXDE'
          return
      except:
        pass
    if wm_name=='Xfwm4':
      self.window_manager = 'XFCE4'
    elif wm_name=='Marco':
      self.window_manager = 'MATE'
    elif wm_name=='Mutter (Muffin)':
      self.window_manager = 'CINNAMON'
    else:
     try:
      output = os.popen("gnome-session --version")
      result = output.readlines()
      version = result[0].split()[1].split('.')[0]
      if version == '2':
        self.window_manager = 'Gnome'
      elif version == '3':
        self.window_manager = 'Gnome3'
     except:
      # If something went wrong, just ignore
      pass

  ## class method to find amount of workspaces in user desktop
  def finding_total_workspaces(self):
    self.total_workspaces = int(self.exec_cmd_under_X("xprop -root _NET_NUMBER_OF_DESKTOPS")[36:38])

    # Save this total_workspaces value
    if not WallpapozSystem.static_finding_total_workspaces_called_p:
      WallpapozSystem.static_finding_total_workspaces_called = True
      WallpapozSystem.static_total_workspaces = self.total_workspaces

  ## class method to find how many rows and columns of workspaces
  def finding_row_and_column(self):
    self.row_workspaces = self.geometry_height / self.screen_height
    self.column_workspaces = self.geometry_width / self.screen_width

  ## class method to find how many workspaces there are in user desktop
  def get_total_workspaces(self):
    return self.total_workspaces

  ## class method to know what workspace we are in now
  def current_desktop(self):
    raw_workspace = self.exec_cmd_under_X('xprop -root _NET_CURRENT_DESKTOP')
    workspace = int(raw_workspace[33] + raw_workspace[34])
    return workspace

  ## class method to change desktop wallpaper
  def change_wallpaper(self, wallpaper):
    if self.window_manager == "Gnome":
      os.system('gconftool-2 -t string -s /desktop/gnome/background/picture_filename ' +
        '"' + wallpaper + '"' + ' -s /desktop/gnome/background/picture_options ' +
        self.wallpaper_style)
    elif self.window_manager == "Gnome3":
      os.system("gsettings set org.gnome.desktop.background picture-uri 'file://" + wallpaper + "'")
      os.system("gsettings set org.gnome.desktop.background picture-options " + self.wallpaper_style)
    elif self.window_manager == "MATE":
      os.system('gsettings set org.mate.background picture-filename ' + '"' + wallpaper + '"')
      os.system('gsettings set org.mate.background picture-options ' + self.wallpaper_style)
    elif self.window_manager == "CINNAMON":
      os.system("gsettings set org.cinnamon.desktop.background picture-uri 'file://" + wallpaper + "'")
      os.system("gsettings set org.cinnamon.desktop.background picture-options " + self.wallpaper_style)
    elif self.window_manager == "XFCE4":
      os.system("xfconf-query -c xfce4-desktop -p /backdrop/screen0/monitor0/image-path -s " +
        '"' + wallpaper + '"')
      os.system("xfconf-query -c xfce4-desktop -p /backdrop/screen0/monitor0/image-style -s " +
        self.wallpaper_style)
    elif self.window_manager == "LXDE":
      os.system("pcmanfm -w " + '"' + wallpaper + '"')
      os.system("pcmanfm --wallpaper-mode=" + self.wallpaper_style)

  ## class method to find current desktop wallpaper
  def finding_current_wallpaper(self):
    if self.window_manager == "Gnome":
      return os.popen("gconftool-2 -g /desktop/gnome/background/picture_filename").read()[:-1]
    elif self.window_manager == "Gnome3":
      return os.popen("gsettings get org.gnome.desktop.background picture-uri").read()[8:-2]
    elif self.window_manager == "MATE":
      return os.popen("gsettings get org.mate.background picture-filename").read()[:-1]
    elif self.window_manager == "CINNAMON":
      return os.popen("gsettings get org.cinnamon.desktop.background picture-uri").read()[8:-2]
    elif self.window_manager == "XFCE4":
      return os.popen("xfconf-query -c xfce4-desktop -p /backdrop/screen0/monitor0/image-path").read()[:-1]
    else:
      return ""

  ## class method to detect that we have changed workspace or not
  def has_changed(self, previous_desktop, cur_desk):
    if previous_desktop != cur_desk:
      # if we move to workspace that we don't use, just ignore
      if cur_desk >= self.total_workspaces:
        return False
      return True
    else:
      return False

  def respawn_system(self):
    new_argv = []
    new_argv.extend(sys.argv)
    os.execvp('daemon_wallpapoz', new_argv)
    print ("os.execvp failed, exiting...")
    sys.exit(1)

  def prevent_multiple_start(self):
    cpid = os.getpid()
    cid = os.getuid()

    wallpapoz_cmd = "pgrep -U %i -f daemon_wallpapoz" %cid
    python_cmd = "pgrep -U %i -f python" %cid

    wallpapoz_id_list = os.popen(wallpapoz_cmd).read().splitlines()
    python_id_list = os.popen(python_cmd).read().splitlines()

    for id_list in wallpapoz_id_list:
      if ( id_list in python_id_list ) and ( int(id_list) != cpid) :
        os.kill(int(id_list), signal.SIGTERM)

